//! The main code for Nuarth.
#![warn(missing_docs, clippy::missing_docs_in_private_items, clippy::unwrap_used)]

use polling::{Event, Poller};

mod state;
mod config;

use state::DelegationState;
fn main() {
    let display = wayland_server::Display::<DelegationState>::new();
    if display.is_err() {
        println!("[FATAL] Cannot load libwayland-server.so, exiting.");
        println!("[HINT] Compile Nuarth with wayland_server's dlopen feature disabled\n[HINT] to not require this library");
        std::process::exit(1);
    }
    
    #[allow(clippy::unwrap_used)] // Error state checked above
    let mut display = display.unwrap();

    let polling_fd = display.backend().poll_fd();

    let poller = Poller::new().expect("failed to create poller for polling file descriptor");

    unsafe { poller.add(&polling_fd, Event::readable(0)).expect("failed to initalize polling file descriptor"); }

    let mut events = polling::Events::new();
    
    loop { // Event loop
        events.clear();
        poller.wait(&mut events, None).expect("failed waiting for wayland events");

        display.dispatch_clients(&mut DelegationState { surfaces: vec![] }).expect("failed to dispatch event handlers for clients");
    }
}
